
//  This file forms part of the GOBLIN C++ Class Library.
//
//  Initially written by Christian Fremuth-Paeger, August 2000
//
//  Copying, compiling, distribution and modification
//  of this source code is permitted only in accordance
//  with the GOBLIN general licence information.

/// \file canvasBuilder.cpp
/// \brief #canvasBuilder class implementation

#include "canvasBuilder.h"


canvasBuilder::canvasBuilder(const abstractMixedGraph &GC)
    throw(ERRejected) :
    managedObject(GC.Context()),
    G(GC), CFG(GC.Context(),true), piG(G.GetPotentials())
{
    label = new char[255];

    sprintf(predColour,"#%06lX",CT.Colour(goblinController::PRED_COLOUR));
    sprintf(inftyColour,"#%06lX",CT.Colour(goblinController::INFTY_COLOUR));

    ExtractParameters();

    if (nodeColourMode==NODES_FLOATING_COLOURS)
    {
        maxNodeColour = 1;
        for (TNode v=0;v<G.N();v++)
        {
            TNode c = G.NodeColour(v);
            if (c>maxNodeColour && c!=NoNode) maxNodeColour = c;
        }
    }

    if (arcColourMode==ARCS_FLOATING_COLOURS)
    {
        maxEdgeColour = 1;
        for (TArc a=0;a<G.M();a++)
        {
            TArc c = G.EdgeColour(2*a);
            if (c>maxEdgeColour && c!=NoArc) maxEdgeColour = c;
        }
    }

    portMode = PORTS_IMPLICIT;

    if (   CFG.layoutModel==LAYOUT_KANDINSKI
        || CFG.layoutModel==LAYOUT_VISIBILITY
//        || CFG.layoutModel==LAYOUT_ORTHO_BIG
       )
    {
        portMode = PORTS_EXPLICIT;
    }

    ComputeBoundingBox();

    CFG.logMeth = 0;

    LogEntry(LOG_MEM,"...Display object instanciated");
}


canvasBuilder::~canvasBuilder() throw()
{
    delete[] label;

    LogEntry(LOG_MEM,"...Display object disallocated");
}


unsigned long canvasBuilder::Allocated() const throw()
{
    return 255;
}


void canvasBuilder::ComputeBoundingBox() throw()
{
    minX =  InfFloat;
    minY =  InfFloat;
    maxX = -InfFloat;
    maxY = -InfFloat;

    for (TNode v=0;v<G.N()+G.NI();v++)
    {
        if (G.HiddenNode(v)) continue;

        TFloat thisX = G.C(v,0);
        TFloat thisY = G.C(v,1);

        if (thisX<minX) minX = thisX;
        if (thisY<minY) minY = thisY;
        if (thisX>maxX) maxX = thisX;
        if (thisY>maxY) maxY = thisY;
    }
}


double canvasBuilder::ScaleDisplayParameters(double zoomCalibration,double xSize, double ySize) throw()
{
    double shrinkingFactor = 1.0;

    zoom = ((CFG.xZoom>CFG.yZoom) ? CFG.yZoom : CFG.xZoom);
    double layoutXRange = maxX-minX+2*CFG.nodeSep;
    double layoutYRange = maxY-minY+2*CFG.nodeSep;

    if (layoutXRange*zoom>xSize)
    {
        shrinkingFactor = (xSize/layoutXRange)/zoom;
        zoom = xSize/layoutXRange;
    }

    if (layoutYRange*zoom>ySize)
    {
        shrinkingFactor = (ySize/layoutYRange)/zoom;
        zoom = ySize/layoutYRange;
    }

    double fontScaleFactor = zoom * 0.002 * CFG.bendSep * CFG.nodeSize / 100.0;
    arcFontSize  = int(7.0 * arcFontSize  / 100.0 * fontScaleFactor);
    nodeFontSize = int(14.0 * nodeFontSize / 100.0 * fontScaleFactor);

    zoom *= zoomCalibration;

    nodeSize  = CFG.nodeSize /100.0 * zoom * CFG.bendSep/5.0;
    arrowSize = CFG.arrowSize/100.0 * zoom;

    if (CFG.bendSep/2.0<CFG.nodeSep/10.0)
    {
        arrowSize *= CFG.bendSep/2.0;
    }
    else
    {
        arrowSize *= CFG.nodeSep/10.0;
    }

    arcLabelSep = CFG.fineSep*zoom;

    if (arcLabelSep<=2*arrowSize) arcLabelSep = 2*arrowSize;

    return shrinkingFactor;
}


long canvasBuilder::PortCX(TNode u,TNode v) throw()
{
    long dx = CanvasCXOfPoint(v)-CanvasCXOfPoint(u);
    long dy = CanvasCYOfPoint(v)-CanvasCYOfPoint(u);
    float norm = sqrt(double(dx*dx+dy*dy));

    if (fabs(norm)<10e-6)
    {
        return CanvasCXOfPoint(v);
    }

    if (nodeShapeMode==NODE_SHAPE_POINT)
    {
        return long(CanvasCXOfPoint(v)-dx*7/norm);
    }
    else
    {
        return long(CanvasCXOfPoint(v)-dx*nodeSize/norm);
    }
}


long canvasBuilder::PortCY(TNode u,TNode v) throw()
{
    long dx = CanvasCXOfPoint(v)-CanvasCXOfPoint(u);
    long dy = CanvasCYOfPoint(v)-CanvasCYOfPoint(u);
    float norm = sqrt(double(dx*dx+dy*dy));

    if (fabs(norm)<10e-6)
    {
        return CanvasCYOfPoint(v);
    }

    if (nodeShapeMode==NODE_SHAPE_POINT)
    {
        return long(CanvasCYOfPoint(v)-dy*5/norm);
    }
    else
    {
        return long(CanvasCYOfPoint(v)-dy*nodeSize/norm);
    }
}


void canvasBuilder::ExtractParameters() throw()
{
    // Assign default values
    arcDisplayMode = TArcDisplayMode(CFG.subgraph);
    arcShapeMode = TArcShapeMode(CFG.arcStyle);
    arcColourMode = TArcColourMode(CFG.arcColours);
    arcWidthMode = ARC_WIDTH_UNIFORM;
    CFG.GetDefaultLayoutParameter(TokLayoutArcWidthMin,arcWidthMin);
    CFG.GetDefaultLayoutParameter(TokLayoutArcWidthMax,arcWidthMax);
    arcStippleMode = ARC_STIPPLE_OFF;
    arcFontType = FONT_HELVETICA_NARROW_OBLIQUE;
    CFG.GetDefaultLayoutParameter(TokLayoutArcFontSize,arcFontSize);

    arrowDisplayMode = TArrowDisplayMode(CFG.arrows / 4);
    arrowPosMode = TArrowPosMode(CFG.arrows % 4);
    arrowSizeRel = CFG.arrowSize;

    nodeShapeMode = TNodeShapeMode(CFG.nodeStyle);
    nodeColourMode = TNodeColourMode(CFG.nodeColours);
    nodeSizeRel = CFG.nodeSize;
    nodeFontType = FONT_HELVETICA_BOLD;
    CFG.GetDefaultLayoutParameter(TokLayoutNodeFontSize,nodeFontSize);

    gridDisplayMode = GRID_DISPLAY_OFF;

    // Overwrite with values found in the pool table
    for (int token=0;token<int(TokLayoutEndSection);++token)
    {
        if (listOfLayoutPars[token].arrayType!=TYPE_INT) continue;
        if (listOfLayoutPars[token].arrayDim!=DIM_SINGLETON) continue;
        if (listOfLayoutPars[token].primaryIndex!=TPoolEnum(token)) continue;

        int* pValue = G.LayoutData()->GetArray<int>(token);

        if (!pValue) continue;

        switch (TOptLayoutTokens(token))
        {
            case TokLayoutArcWidthMode:
            {
                arcWidthMode = TArcWidthMode(*pValue);
                break;
            }
            case TokLayoutArcWidthMin:
            {
                arcWidthMin = *pValue;
                break;
            }
            case TokLayoutArcWidthMax:
            {
                arcWidthMax = *pValue;
                break;
            }
            case TokLayoutArcStippleMode:
            {
                arcStippleMode = TArcStippleMode(*pValue);
                break;
            }
            case TokLayoutNodeFontSize:
            {
                nodeFontSize = *pValue;
                break;
            }
            case TokLayoutArcFontSize:
            {
                arcFontSize = *pValue;
                break;
            }
            case TokLayoutNodeFontType:
            {
                nodeFontType = TXFigFontType(*pValue);
                break;
            }
            case TokLayoutArcFontType:
            {
                arcFontType = TXFigFontType(*pValue);
                break;
            }
            case TokLayoutGridDisplayMode:
            {
                gridDisplayMode = TGridDisplayMode(*pValue);
                break;
            }
            default:
            {
            }
        }
    }

    arcWidthMin = (arcWidthMin<1) ? 1 : arcWidthMin;
    arcWidthMax = (arcWidthMax>arcWidthMin) ? arcWidthMax : arcWidthMin;
}


void canvasBuilder::DisplayGraph() throw(ERRejected)
{
    #if defined(_FAILSAVE_)

    if (G.Dim()==0)
    {
        G.Error(ERR_REJECTED,"canvasBuilder","Missing geometric embedding");
    }

    #endif

    DisplayGrids();

    for (TArc a=0;a<G.M();a++)
    {
        TArc a2 = 2*a;

        if (G.HiddenArc(a2)) continue;

        if (G.Blocking(a2) && G.Blocking(a2+1)) continue;

        if (G.UCap(a2)==0) continue;

        if (G.StartNode(a2)==G.EndNode(a2) && G.ArcLabelAnchor(a2)==NoNode) continue;

        DisplayArc(a);
    }

    for (TNode v=0;v<G.N();v++)
    {
        if (G.HiddenNode(v)) continue;

        DisplayNode(v);
    }

    if (CFG.legenda>0)
    {
        long xm = long(CanvasCX((minX+maxX)/2.0));
        long ym = long(CanvasCY(maxY+CFG.legenda));

        DisplayLegenda(xm,ym,long(CFG.nodeSep*zoom));
    }
}


void canvasBuilder::DisplayArc(TArc a) throw()
{
    TNode u = G.StartNode(2*a);
    TNode v = G.EndNode(2*a);

    bool isPredecessorArc = (G.Pred(v)==2*a || G.Pred(u)==(2*a+1));

    bool drawSub = true;
    TFloat thisSub = fabs(G.Sub(2*a));
    bool isSubgraphArc = (thisSub>CT.epsilon);

    if (arcDisplayMode==ARC_DISPLAY_HIDE_ALL)
    {
        drawSub = false;
    }
    else if (arcDisplayMode==ARC_DISPLAY_SUBGRAPH)
    {
        drawSub = isSubgraphArc;
    }
    else if (arcDisplayMode==ARC_DISPLAY_PREDECESSORS)
    {
        drawSub = isPredecessorArc;
    }
    else if (arcDisplayMode==ARC_DISPLAY_PRED_SUB)
    {
        drawSub = isPredecessorArc | isSubgraphArc;
    }

    if (!drawSub) return;

    TArrowDir displayedArrows = ArrowDirections(a);
    TArrowDir displayedExteriorArrows = ARROW_NONE;
    TArrowDir displayedCenteredArrows = ARROW_NONE;

    if (arrowPosMode==ARROWS_CENTERED)
    {
        if (displayedArrows!=ARROW_BOTH)
        {
            displayedCenteredArrows = displayedArrows;
        }
    }
    else
    {
        displayedExteriorArrows = displayedArrows;
    }

    if (drawSub)
    {
        int width = arcWidthMin;

        if (arcWidthMode==ARC_WIDTH_PREDECESSORS)
        {
            if (isPredecessorArc)
            {
                width = arcWidthMax;
            }
        }
        else if (arcWidthMode==ARC_WIDTH_SUBGRAPH)
        {
            if (isSubgraphArc)
            {
                width = arcWidthMax;
            }
        }
        else if (arcWidthMode==ARC_WIDTH_FLOW_LINEAR)
        {
            width += int(thisSub*(arcWidthMax-arcWidthMin));
        }
        else if (arcWidthMode==ARC_WIDTH_FLOW_LOGARITHMIC)
        {
            width += int(log(thisSub+1.0)*(arcWidthMax-arcWidthMin));
        }
        else if (arcWidthMode==ARC_WIDTH_EMPTY_FREE_FULL)
        {
            if (thisSub<G.LCap(2*a)+CT.epsilon)
            {
                width = arcWidthMin;
            }
            else if (thisSub<G.UCap(2*a)-CT.epsilon)
            {
                width = (arcWidthMin+arcWidthMax)/2;
            }
            else
            {
                width = arcWidthMax;
            }
        }

        TLineStyle lineStyle = LINE_STYLE_SOLID;

        if (arcStippleMode==ARC_STIPPLE_PREDECESSORS)
        {
            if (isPredecessorArc)
            {
                lineStyle = LINE_STYLE_DOT;
            }
        }
        else if (arcStippleMode==ARC_STIPPLE_FREE)
        {
            if (   thisSub>G.LCap(2*a)+CT.epsilon
                && thisSub<G.UCap(2*a)-CT.epsilon
               )
            {
                lineStyle = LINE_STYLE_DOT;
            }
        }
        else if (arcStippleMode==ARC_STIPPLE_FRACTIONAL)
        {
            if (floor(thisSub)!=thisSub)
            {
                lineStyle = LINE_STYLE_DOT;
            }
        }
        else if (arcStippleMode==ARC_STIPPLE_COLOURS)
        {
            if (G.EdgeColour(2*a)!=NoArc)
            {
                lineStyle = TLineStyle(int(G.EdgeColour(2*a))%4);
                width = arcWidthMin+int(G.EdgeColour(2*a))/4*(arcWidthMax-arcWidthMin);
            }
        }

        TIndex colourIndex = goblinController::OUTLINE_COLOUR;

        if (arcColourMode==ARCS_REPEAT_COLOURS)
        {
            colourIndex = goblinController::ZERO_COLOUR
                        + G.EdgeColour(2*a)%(goblinController::MAX_COLOUR -goblinController::ZERO_COLOUR);
        }
        else if (arcColourMode==ARCS_COLOUR_PRED)
        {
            if (isPredecessorArc)
            {
                colourIndex = goblinController::PRED_COLOUR;
            }
        }
        else if (arcColourMode!=ARCS_UNCOLOURED)
        {
            colourIndex = goblinController::ZERO_COLOUR + G.EdgeColour(2*a);
        }

        WriteArc(a,u,v,lineStyle,width,displayedExteriorArrows,colourIndex,100);

        if (displayedCenteredArrows!=ARROW_NONE)
        {
            TNode y = G.PortNode(2*a);
            TNode x = u;

            while (y!=NoNode)
            {
                if (portMode==PORTS_IMPLICIT || x!=u)
                {
                    if (displayedCenteredArrows==ARROW_FORWARD) DisplayArrow(a,x,y);
                    if (displayedCenteredArrows==ARROW_BACKWARD) DisplayArrow(a,y,x);
                }

                x = y;
                y = G.ThreadSuccessor(y);
            }

            if (x!=v && portMode==PORTS_IMPLICIT)
            {
                if (displayedCenteredArrows==ARROW_FORWARD) DisplayArrow(a,x,v);
                if (displayedCenteredArrows==ARROW_BACKWARD) DisplayArrow(a,v,x);
            }
        }
    }

    if (   (u!=v || G.ArcLabelAnchor(2*a)!=NoNode)
        && (   CFG.arcLabels || CFG.arcLabels2 || CFG.arcLabels3
            || strcmp(CFG.arcFormatting,"")
           )
       )
    {
        DisplayArcLabel(a,u,v,displayedCenteredArrows);
    }
}


void canvasBuilder::WriteArc(TArc a,TNode u,TNode v,TLineStyle lineStyle,int width,
    TArrowDir displayedArrows,TIndex colourIndex,int depth) throw()
{
    if (lineStyle==LINE_STYLE_SOLID)
    {
        WriteLine(a,u,v,lineStyle,width,displayedArrows,colourIndex,depth);
    }
    else
    {
        if (arcColourMode!=ARCS_UNCOLOURED)
        {
            WriteLine(a,u,v,LINE_STYLE_SOLID,width,displayedArrows,colourIndex,depth+1);
        }
        else
        {
            WriteLine(a,u,v,LINE_STYLE_SOLID,width,displayedArrows,
                goblinController::ZERO_COLOUR + 3,depth+1);
        }

        WriteLine(a,u,v,lineStyle,width,displayedArrows,
            goblinController::OUTLINE_COLOUR,depth);
    }
}


canvasBuilder::TArrowDir canvasBuilder::ArrowDirections(TArc a) throw()
{
    switch (arrowDisplayMode)
    {
        case ARROWS_OFF:
        {
            return ARROW_NONE;
        }
        case ARROWS_IMPL_ORIENTATION:
        {
            return ARROW_FORWARD;
        }
        case ARROWS_FLOW_DIRECTION:
        {
            if (G.Sub(2*a)>CT.epsilon)
            {
                return ARROW_FORWARD;
            }
            else if (G.Sub(2*a)<-CT.epsilon)
            {
                return ARROW_BACKWARD;
            }
            else
            {
                return ARROW_NONE;
            }
        }
        case ARROWS_LIKE_PREDECESSORS:
        {
            if (G.Pred(G.EndNode(2*a))==2*a)
            {
                return ARROW_FORWARD;
            }
            else if (G.Pred(G.EndNode(2*a+1))==2*a+1)
            {
                return ARROW_BACKWARD;
            }
            else
            {
                return ARROW_NONE;
            }
        }
        default:
        case ARROWS_ARC_ORIENTATION:
        {
            return TArrowDir(G.Orientation(2*a));
        }
    }
}


void canvasBuilder::DisplayArrow(TArc a,TNode u,TNode v) throw()
{
    double dx = CanvasCXOfPoint(v)-CanvasCXOfPoint(u);
    double dy = CanvasCYOfPoint(v)-CanvasCYOfPoint(u);
    double norm = sqrt(dx*dx+dy*dy);

    if (fabs(norm)<CFG.bendSep*0.5) return;

    dx = dx/norm;
    dy = dy/norm;

    WriteArrow(a,
        long((CanvasCXOfPoint(u)+CanvasCXOfPoint(v))/2+arrowSize*dx),
        long((CanvasCYOfPoint(u)+CanvasCYOfPoint(v))/2+arrowSize*dy),
        dx,dy);
}


void canvasBuilder::DisplayArcLabel(TArc a,TNode u,TNode v,TArrowDir displayedCenteredArrows) throw()
{
    long xm = 0;
    long ym = 0;
    TNode p = G.ArcLabelAnchor(2*a);

    if (p==NoNode)
    {
        double dx = CanvasCXOfPoint(v)-CanvasCXOfPoint(u);
        double dy = CanvasCYOfPoint(v)-CanvasCYOfPoint(u);
        double norm = sqrt(dx*dx+dy*dy);

        if (fabs(norm)<CFG.bendSep*0.5) return;

        dx = dx/norm;
        dy = dy/norm;
        xm = long((CanvasCXOfPoint(u)+CanvasCXOfPoint(v))/2+arcLabelSep*dy);
        ym = long((CanvasCYOfPoint(u)+CanvasCYOfPoint(v))/2-arcLabelSep*dx);
    }
    else
    {
        xm = CanvasCXOfPoint(p);
        ym = CanvasCYOfPoint(p);
    }

    WriteArcLabel(a,xm,ym);
}


void canvasBuilder::ComposeArcLabel(TArc a,ofstream& expFile) throw()
{
    if (strcmp(CFG.arcFormatting,"")==0)
    {
        bool first = true;

        if (CFG.arcLabels)
        {
            expFile << ArcLabel(a,CFG.arcLabels);
            first = false;
        }

        if (CFG.arcLabels2)
        {
            if (!first) expFile << "/";

            expFile << ArcLabel(a,CFG.arcLabels2);
            first = false;
        }

        if (CFG.arcLabels3)
        {
            if (!first) expFile << "/";

            expFile << ArcLabel(a,CFG.arcLabels3);
        }
    }
    else 
    {
        for (unsigned i=0;i<strlen(CFG.arcFormatting);i++)
        {
            if (CFG.arcFormatting[i]=='#' && i<strlen(CFG.arcFormatting)-1)
            {
                expFile << ArcLabel(a,int(CFG.arcFormatting[i+1]-'0'));
                i++;
                continue;
            }

            if (CFG.arcFormatting[i]!='%' || i==strlen(CFG.arcFormatting)-1)
            {
                expFile << char(CFG.arcFormatting[i]);
                continue;
            }

            switch (CFG.arcFormatting[i+1])
            {
                case '1':
                {
                    expFile << ArcLabel(a,CFG.arcLabels);
                    i++;
                    break;
                }
                case '2':
                {
                    expFile << ArcLabel(a,CFG.arcLabels2);
                    i++;
                    break;
                }
                case '3':
                {
                    expFile << ArcLabel(a,CFG.arcLabels3);
                    i++;
                    break;
                }
                default:
                {
                    expFile << char(CFG.arcFormatting[i]);
                }
            }
        }
    }
}


char* canvasBuilder::ArcLabel(TArc a,int option) throw(ERRejected)
{
    switch (option)
    {
        case 0:
        {
            strcpy(label,"");
            break;
        }
        case 1:
        {
            sprintf(label,"%ld",a);
            break;
        }
        case 2:
        {
            if (G.UCap(2*a)==InfCap) return "*";
            sprintf(label,"%g",G.UCap(2*a));
            break;
        }
        case 3:
        {
            sprintf(label,"%g",fabs(G.Sub(2*a)));
            break;
        }
        case 4:
        {
            if (G.Length(2*a)==InfFloat || G.Length(2*a)==-InfFloat) return "*";
            sprintf(label,"%g",G.Length(2*a));
            break;
        }
        case 5:
        {
            sprintf(label,"%g",G.RedLength(piG,2*a));
            break;
        }
        case 6:
        {
            sprintf(label,"%g",G.LCap(2*a));
            break;
        }
        case 7:
        {
            sprintf(label,"%ld",a+1);
            break;
        }
        case 8:
        {
            if (G.EdgeColour(2*a)==NoArc) sprintf(label,"*");
            else sprintf(label,"%ld",G.EdgeColour(2*a));
            break;
        }
        default:
        {
            UnknownOption("ArcLabel",option);
        }
    }

    return label;
}


void canvasBuilder::ComposeNodeLabel(TNode v,ofstream& expFile) throw()
{
    if (strcmp(CFG.nodeFormatting,"")==0)
    {
        expFile << NodeLabel(v,CFG.nodeLabels);
    }
    else
    {
        for (unsigned i=0;i<strlen(CFG.nodeFormatting);i++)
        {
            if (CFG.nodeFormatting[i]=='#' && i<strlen(CFG.nodeFormatting)-1)
            {
                expFile << NodeLabel(v,int(CFG.nodeFormatting[i+1]-'0'));
                i++;
                continue;
            }

            if (CFG.nodeFormatting[i]!='%' || i==strlen(CFG.nodeFormatting)-1)
            {
                expFile << char(CFG.nodeFormatting[i]);
                continue;
            }

            switch (CFG.nodeFormatting[i+1])
            {
                case '1':
                {
                    expFile << NodeLabel(v,CFG.nodeLabels);
                    i++;
                    break;
                }
                default:
                {
                    expFile << char(CFG.nodeFormatting[i]);
                }
            }
        }
    }
}


char* canvasBuilder::NodeLabel(TNode v,int option) throw(ERRejected)
{
    switch (option)
    {
        case 0:
        {
            strcpy(label,"");
            break;
        }
        case 1:
        {
            sprintf(label,"%ld",v);
            break;
        }
        case 2:
        {
            if (G.Dist(v)==InfFloat || G.Dist(v)==-InfFloat) return "*";
            sprintf(label,"%g",G.Dist(v));
            break;
        }
        case 3:
        {
            if (G.Pi(v)==InfFloat || G.Pi(v)==-InfFloat) return "*";
            sprintf(label,"%g",G.Pi(v));
            break;
        }
        case 4:
        {
            if (G.NodeColour(v)==NoNode) return "*";
            sprintf(label,"%ld",G.NodeColour(v));
            break;
        }
        case 5:
        {
            sprintf(label,"%g",G.Demand(v));
            break;
        }
        case 6:
        {
            sprintf(label,"%ld",v+1);
            break;
        }
        default:
        {
            UnknownOption("NodeLabel",option);
        }
    }

    return label;
}


char* canvasBuilder::ArcLegenda(int option) throw(ERRejected)
{
    switch (option)
    {
        case 0:
        {
            strcpy(label,"");
            break;
        }
        case 1:
        case 7:
        {
            sprintf(label,"a");
            break;
        }
        case 2:
        {
            sprintf(label,"ucap");
            break;
        }
        case 3:
        {
            sprintf(label,"x");
            break;
        }
        case 4:
        {
            sprintf(label,"length");
            break;
        }
        case 5:
        {
            sprintf(label,"redlength");
            break;
        }
        case 6:
        {
            sprintf(label,"lcap");
            break;
        }
        case 8:
        {
            sprintf(label,"colour");
            break;
        }
        default:
        {
            UnknownOption("ArcLegenda",option);
        }
    }

    return label;
}


char* canvasBuilder::NodeLegenda(char* index,int option) throw(ERRejected)
{
    switch (option)
    {
        case 1:
        case 6:
        {
            sprintf(label,"%s",index);
            break;
        }
        case 2:
        {
            sprintf(label,"d(%s)",index);
            break;
        }
        case 3:
        {
            sprintf(label,"pi(%s)",index);
            break;
        }
        case 4:
        {
            sprintf(label,"colour(%s)",index);
            break;
        }
        case 5:
        {
            sprintf(label,"B(%s)",index);
            break;
        }
        default:
        {
            UnknownOption("NodeLegenda",option);
        }
    }

    return label;
}


char* canvasBuilder::FixedNodeColour(TIndex c) throw()
{
    if (c==NoNode)
    {
        sprintf(nodeColour,"#%06lX",CT.Colour(goblinController::NO_COLOUR));
    }
    else
    {
        TNode c0 = goblinController::ZERO_COLOUR + c;

        sprintf(nodeColour,"#%06lX",CT.Colour(c0));
    }

    return nodeColour;
}


char* canvasBuilder::FixedEdgeColour(TIndex c) throw()
{
    if (c==NoArc)
    {
        // Other than for node colours, NO_COLOUR does not apply since this is
        // displayed white. Use some kind of grey here
        sprintf(nodeColour,"#%06lX",CT.Colour(goblinController::MAX_COLOUR));
    }
    else
    {
        TNode c0 = goblinController::ZERO_COLOUR + c;

        sprintf(nodeColour,"#%06lX",CT.Colour(c0));
    }

    return nodeColour;
}


char* canvasBuilder::SmoothColour(TIndex c,TIndex maxColour,goblinController::TColourDummy undefColour)
    throw()
{
    if (c>maxColour)
    {
        sprintf(nodeColour,"#%06lX",CT.Colour(undefColour));
        return nodeColour;
    }

    double ratio = double(c)/double(maxColour+1);

    long unsigned rSat = 100;
    long unsigned bSat = 100;
    long unsigned gSat = 100;

    if (ratio<0.333)
    {
        rSat = (long unsigned)(ceil(255*3*(0.333-ratio)));
        gSat = (long unsigned)(ceil(255*3*ratio));
    }
    else if (ratio<0.667)
    {
        gSat = (long unsigned)(ceil(254*3*(0.667-ratio)));
        bSat = (long unsigned)(ceil(254*3*(ratio-0.333)));
    }
    else
    {
        bSat = (long unsigned)(ceil(255*3*(1-ratio)));
        rSat = (long unsigned)(ceil(255*3*(ratio-0.667)));
    }

    sprintf(nodeColour,"#%02lX%02lX%02lX",rSat,gSat,bSat);

    return nodeColour;
}


const char* unixFontType[] =
{
    "times-medium-r-normal-", //    FONT_TIMES_ROMAN = 0,
    "times-medium-i-normal-", //    FONT_TIMES_ITALIC = 1,
    "times-bold-r-normal-", //    FONT_TIMES_BOLD = 2,
    "times-bold-i-normal-", //    FONT_TIMES_BOLD_ITALIC = 3,
    "itc avant garde-medium-r-normal-sans", //    FONT_AVANTGARDE_BOOK = 4,
    "itc avant garde-medium-o-normal-sans", //    FONT_AVANTGARDE_BOOK_OBLIQUE = 5,
    "itc avant garde-demi-r-normal-sans",   //    FONT_AVANTGARDE_DEMI = 6,
    "itc avant garde-demi-o-normal-sans",   //    FONT_AVANTGARDE_DEMI_OBLIQUE = 7,
    "itc bookman-light-r-normal-", //    FONT_BOOKMAN_LIGHT = 8,
    "itc bookman-light-i-normal-", //    FONT_BOOKMAN_LIGHT_ITALIC = 9,
    "itc bookman l-medium-r-normal-", //    FONT_BOOKMAN_DEMI = 10,
    "itc bookman l-medium-i-normal-", //    FONT_BOOKMAN_DEMI_ITALIC = 11,
    "courier-medium-r-normal-",   //    FONT_COURIER = 12,
    "courier-medium-o-normal-",   //    FONT_COURIER_OBLIQUE = 13,
    "courier-bold-r-normal-",     //    FONT_COURIER_BOLD = 14,
    "courier-bold-o-normal-",     //    FONT_COURIER_BOLD_OBLIQUE = 15,
    "helvetica-medium-r-normal-", //    FONT_HELVETICA = 16,
    "helvetica-medium-o-normal-", //    FONT_HELVETICA_OBLIQUE = 17,
    "helvetica-bold-r-normal-",   //    FONT_HELVETICA_BOLD = 18,
    "helvetica-bold-o-normal-",   //    FONT_HELVETICA_BOLD_OBLIQUE = 19,
    "helvetica-medium-r-narrow-", //    FONT_HELVETICA_NARROW = 20,
    "helvetica-medium-o-narrow-", //    FONT_HELVETICA_NARROW_OBLIQUE = 21,
    "helvetica-bold-r-narrow-",   //    FONT_HELVETICA_NARROW_BOLD = 22,
    "helvetica-bold-o-narrow-",   //    FONT_HELVETICA_NARROW_BOLD_OBLIQUE = 23,
    "new century schoolbook-medium-r-normal-", //    FONT_NEW_CENTURY_SCHOOLBOOK_ROMAN = 24,
    "new century schoolbook-medium-i-normal-", //    FONT_NEW_CENTURY_SCHOOLBOOK_ITALIC = 25,
    "new century schoolbook-bold-r-normal-",   //    FONT_NEW_CENTURY_SCHOOLBOOK_BOLD = 26,
    "new century schoolbook-bold-i-normal-",   //    FONT_NEW_CENTURY_SCHOOLBOOK_BOLD_ITALIC = 27,
    "palatino-medium-r-normal-", //    FONT_PALATINO_ROMAN = 28,
    "palatino-medium-i-normal-", //    FONT_PALATINO_ITALIC = 29,
    "palatino-bold-r-normal-",   //    FONT_PALATINO_BOLD = 30,
    "palatino-bold-i-normal-",   //    FONT_PALATINO_BOLD_ITALIC = 31,
    "symbol-medium-r-normal-",   //    FONT_SYMBOL = 32,
    "itc zapf chancery-medium-i-normal-", //    FONT_ZAPF_CHANCERY_MEDIUM_ITALIC = 33,
    "itc zapf dingbats-medium-r-normal-"  //    FONT_ZAPF_DINGBATS = 34
};

